/**
 * @module-suite/create-url v0.2.0
 * Author: Gabe M <hey@gabe.mx>
 *
 * https://github.com/zelzen/module-suite
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = require('path');
var qs = _interopDefault(require('qs'));
var semver = _interopDefault(require('semver'));

/**
 * Creates an import for Module Server
 */
function createImport(moduleName, moduleVersion, _ref) {
  var filePath = _ref.filePath,
      minify = _ref.minify,
      host = _ref.host,
      output = _ref.output,
      transforms = _ref.transforms;

  if (!host || host.startsWith('http') === false) {
    throw new Error("Expected host to be a valid URL. Received \"" + host + "\"");
  }

  var isValidSemver = semver.valid(moduleVersion) != null || semver.validRange(moduleVersion) != null;

  if (isValidSemver === false) {
    throw new Error("Expected valid semver or range for \"" + moduleName + "\". Received: " + moduleVersion);
  }

  var queryParams = qs.stringify({
    minify: minify,
    output: output,
    // Pass `transforms=false` if an empty Array is specified.
    transforms: transforms != null && transforms.length === 0 ? false : transforms
  }, {
    arrayFormat: 'comma'
  });
  var path$1 = "/" + moduleName + "@" + moduleVersion;
  if (filePath) path$1 += "/" + filePath;
  if (queryParams) path$1 += "?" + queryParams;
  return host + path.normalize(path$1);
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function createDependencyImports(deps, _ref) {
  var _ref$filePaths = _ref.filePaths,
      filePaths = _ref$filePaths === void 0 ? {} : _ref$filePaths,
      options = _objectWithoutPropertiesLoose(_ref, ["filePaths"]);

  return Object.entries(deps).reduce(function (accumDeps, _ref2) {
    var moduleName = _ref2[0],
        moduleVersion = _ref2[1];
    var importDec = createImport(moduleName, moduleVersion, _extends({}, options, {
      filePath: filePaths[moduleName]
    })); // @ts-ignore

    accumDeps[moduleName] = importDec;
    return accumDeps;
  }, {});
}

exports.createDependencyImports = createDependencyImports;
exports.createImport = createImport;
